@@ -14,6 +14,7 @@
 import argparse
 import datetime
 import functools
+import itertools
 import logging
 import math
 import random
@@ -22,11 +23,14 @@ import types
 
 from cliff import show
 import futurist
-from oslo_utils import timeutils
+import iso8601
+from monotonic import monotonic as now  # noqa
 import six.moves
 
+from gnocchiclient import utils
 from gnocchiclient.v1 import metric_cli
 
+
 LOG = logging.getLogger(__name__)
 
 
@@ -36,16 +40,10 @@ def _pickle_method(m):
     else:
         return getattr, (m.im_self, m.im_func.func_name)
 
+
 six.moves.copyreg.pickle(types.MethodType, _pickle_method)
 
 
-def grouper(iterable, n, fillvalue=None):
-    "Collect data into fixed-length chunks or blocks"
-    # grouper('ABCDEFG', 3, 'x') --> ABC DEF
-    args = [iter(iterable)] * n
-    return six.moves.zip(*args)
-
-
 def _positive_non_zero_int(argument_value):
     if argument_value is None:
         return None
@@ -60,26 +58,31 @@ def _positive_non_zero_int(argument_value):
     return value
 
 
+class StopWatch(object):
+    def __init__(self):
+        self.started_at = now()
+
+    def elapsed(self):
+        return max(0.0, now() - self.started_at)
+
+
 def measure_job(fn, *args, **kwargs):
-    # NOTE(sileht): This is not part of BenchmarkPool
     # because we cannot pickle BenchmarkPool class
-    sw = timeutils.StopWatch().start()
+    sw = StopWatch()
     return fn(*args, **kwargs), sw.elapsed()
 
 
 class BenchmarkPool(futurist.ProcessPoolExecutor):
     def submit_job(self, times, fn, *args, **kwargs):
-        self.sw = timeutils.StopWatch()
-        self.sw.start()
+        self.sw = StopWatch()
         self.times = times
         return [self.submit(measure_job, fn, *args, **kwargs)
                 for i in six.moves.range(times)]
 
     def map_job(self, fn, iterable, **kwargs):
-        self.sw = timeutils.StopWatch()
         r = []
         self.times = 0
-        self.sw.start()
+        self.sw = StopWatch()
         for item in iterable:
             r.append(self.submit(measure_job, fn, item, **kwargs))
             self.times += 1
@@ -115,9 +118,12 @@ class BenchmarkPool(futurist.ProcessPoolExecutor):
         return results, runtime, {
             'client workers': self._max_workers,
             verb + ' runtime': "%.2f seconds" % runtime,
+            verb + ' runtime (cumulated)': "%.2f seconds" % sum(latencies),
             verb + ' executed': self.statistics.executed,
             verb + ' speed': (
-                "%.2f %s/s" % (self.statistics.executed / runtime, verb)
+                "%.2f %s/s" % ((self.statistics.executed * self._max_workers
+                                / sum(latencies))
+                               if runtime != 0 else 0, verb)
             ),
             verb + ' failures': self.statistics.failures,
             verb + ' failures rate': (
@@ -179,7 +185,7 @@ class CliBenchmarkMetricShow(CliBenchmarkBase,
     def take_action(self, parsed_args):
         pool = BenchmarkPool(parsed_args.workers)
         LOG.info("Getting metrics")
-        futures = pool.map_job(self.app.client.metric.get,
+        futures = pool.map_job(utils.get_client(self).metric.get,
                                parsed_args.metric * parsed_args.count,
                                resource_id=parsed_args.resource_id)
         result, runtime, stats = pool.wait_job("show", futures)
@@ -201,19 +207,21 @@ class CliBenchmarkMetricCreate(CliBenchmarkBase,
                             help="Keep created metrics")
         return parser
 
-    def _take_action(self, metric, parsed_args):
+    def take_action(self, parsed_args):
         pool = BenchmarkPool(parsed_args.workers)
 
         LOG.info("Creating metrics")
-        futures = pool.submit_job(parsed_args.count,
-                                  self.app.client.metric.create,
-                                  metric, refetch_metric=False)
+        futures = pool.submit_job(
+            parsed_args.count,
+            utils.get_client(self).metric._create_new,
+            archive_policy_name=parsed_args.archive_policy_name,
+            resource_id=parsed_args.resource_id)
         created_metrics, runtime, stats = pool.wait_job("create", futures)
 
         if not parsed_args.keep:
             LOG.info("Deleting metrics")
             pool = BenchmarkPool(parsed_args.workers)
-            futures = pool.map_job(self.app.client.metric.delete,
+            futures = pool.map_job(utils.get_client(self).metric.delete,
                                    [m['id'] for m in created_metrics])
             _, runtime, dstats = pool.wait_job("delete", futures)
             stats.update(dstats)
@@ -237,13 +245,14 @@ class CliBenchmarkMeasuresAdd(CliBenchmarkBase,
                             help="Number of measures to send in each batch")
         parser.add_argument("--timestamp-start", "-s",
                             default=(
-                                timeutils.utcnow(True)
+                                datetime.datetime.now(tz=iso8601.iso8601.UTC)
                                 - datetime.timedelta(days=365)),
-                            type=timeutils.parse_isotime,
+                            type=utils.parse_date,
                             help="First timestamp to use")
         parser.add_argument("--timestamp-end", "-e",
-                            default=timeutils.utcnow(True),
-                            type=timeutils.parse_isotime,
+                            default=(
+                                datetime.datetime.now(tz=iso8601.iso8601.UTC)),
+                            type=utils.parse_date,
                             help="Last timestamp to use")
         parser.add_argument("--wait",
                             default=False,
@@ -265,7 +274,7 @@ class CliBenchmarkMeasuresAdd(CliBenchmarkBase,
 
         start = int(parsed_args.timestamp_start.strftime("%s"))
         end = int(parsed_args.timestamp_end.strftime("%s"))
-        count = parsed_args.count
+        count = parsed_args.batch
 
         if (end - start) < count:
             raise ValueError(
@@ -274,43 +283,43 @@ class CliBenchmarkMeasuresAdd(CliBenchmarkBase,
 
         random_values = (random.randint(- 2 ** 32, 2 ** 32)
                          for _ in six.moves.range(count))
-        all_measures = ({"timestamp": ts, "value": v}
-                        for ts, v
-                        in six.moves.zip(
-                            six.moves.range(start,
-                                            end,
-                                            (end - start) // count),
-                            random_values))
-
-        measures = grouper(all_measures, parsed_args.batch)
+        measures = [{"timestamp": ts, "value": v}
+                    for ts, v
+                    in six.moves.zip(
+                        six.moves.range(start,
+                                        end,
+                                        (end - start) // count),
+                        random_values)]
 
+        times = parsed_args.count // parsed_args.batch
         futures = pool.map_job(functools.partial(
-            self.app.client.metric.add_measures,
-            parsed_args.metric), measures, resource_id=parsed_args.resource_id)
+            utils.get_client(self).metric.add_measures,
+            parsed_args.metric), itertools.repeat(measures, times),
+            resource_id=parsed_args.resource_id)
         _, runtime, stats = pool.wait_job("push", futures)
 
         stats['measures per request'] = parsed_args.batch
         stats['measures push speed'] = (
             "%.2f push/s" % (
-                parsed_args.batch * pool.statistics.executed / runtime
+                parsed_args.batch * float(stats['push speed'][:-7])
             )
         )
 
         if parsed_args.wait:
-            with timeutils.StopWatch() as sw:
-                while True:
-                    status = self.app.client.status.get()
-                    remaining = int(status['storage']['summary']['measures'])
-                    if remaining == 0:
-                        stats['extra wait to process measures'] = (
-                            "%s seconds" % sw.elapsed()
-                        )
-                        break
-                    else:
-                        LOG.info(
-                            "Remaining measures to be processed: %d"
-                            % remaining)
-                    time.sleep(1)
+            sw = StopWatch()
+            while True:
+                status = utils.get_client(self).status.get()
+                remaining = int(status['storage']['summary']['measures'])
+                if remaining == 0:
+                    stats['extra wait to process measures'] = (
+                        "%s seconds" % sw.elapsed()
+                    )
+                    break
+                else:
+                    LOG.info(
+                        "Remaining measures to be processed: %d"
+                        % remaining)
+                time.sleep(1)
 
         return self.dict2columns(stats)
 
@@ -331,11 +340,12 @@ class CliBenchmarkMeasuresShow(CliBenchmarkBase,
         pool = BenchmarkPool(parsed_args.workers)
         LOG.info("Getting measures")
         futures = pool.submit_job(parsed_args.count,
-                                  self.app.client.metric.get_measures,
+                                  utils.get_client(self).metric.get_measures,
                                   metric=parsed_args.metric,
                                   resource_id=parsed_args.resource_id,
                                   aggregation=parsed_args.aggregation,
                                   start=parsed_args.start,
                                   stop=parsed_args.stop)
         result, runtime, stats = pool.wait_job("show", futures)
+        stats['measures per request'] = len(result[0])
         return self.dict2columns(stats)
