@@ -10,6 +10,9 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
+import distutils.util
+
 from cliff import command
 from cliff import lister
 from cliff import show
@@ -47,23 +50,27 @@ class CliResourceList(lister.Lister):
                             default="generic", help="Type of resource")
         return parser
 
+    def _list2cols(self, resources):
+        """Return a formatted list of resources."""
+        if not resources:
+            return self.COLS, []
+        cols = list(self.COLS)
+        for k in resources[0]:
+            if k not in cols:
+                cols.append(k)
+        if 'creator' in cols:
+            cols.remove('created_by_user_id')
+            cols.remove('created_by_project_id')
+        return utils.list2cols(cols, resources)
+
     def take_action(self, parsed_args):
         resources = utils.get_client(self).resource.list(
             resource_type=parsed_args.resource_type,
-            **self._get_pagination_options(parsed_args))
-        return utils.list2cols(self.COLS, resources)
-
-    @staticmethod
-    def _get_pagination_options(parsed_args):
-        options = dict(
-            details=parsed_args.details,
-            sorts=parsed_args.sort,
-            limit=parsed_args.limit,
-            marker=parsed_args.marker)
-
-        if hasattr(parsed_args, 'history'):
-            options['history'] = parsed_args.history
-        return options
+            **utils.get_pagination_options(parsed_args))
+        # Do not dump metrics because it makes the list way too long
+        for r in resources:
+            del r['metrics']
+        return self._list2cols(resources)
 
 
 class CliResourceHistory(CliResourceList):
@@ -80,11 +87,10 @@ class CliResourceHistory(CliResourceList):
         resources = utils.get_client(self).resource.history(
             resource_type=parsed_args.resource_type,
             resource_id=parsed_args.resource_id,
-            **self._get_pagination_options(parsed_args))
-        cols = resources[0].keys() if resources else self.COLS
+            **utils.get_pagination_options(parsed_args))
         if parsed_args.formatter == 'table':
-            return utils.list2cols(cols, map(normalize_metrics, resources))
-        return utils.list2cols(cols, resources)
+            return self._list2cols(list(map(normalize_metrics, resources)))
+        return self._list2cols(resources)
 
 
 class CliResourceSearch(CliResourceList):
@@ -99,8 +105,11 @@ class CliResourceSearch(CliResourceList):
         resources = utils.get_client(self).resource.search(
             resource_type=parsed_args.resource_type,
             query=parsed_args.query,
-            **self._get_pagination_options(parsed_args))
-        return utils.list2cols(self.COLS, resources)
+            **utils.get_pagination_options(parsed_args))
+        # Do not dump metrics because it makes the list way too long
+        for r in resources:
+            del r['metrics']
+        return self._list2cols(resources)
 
 
 def normalize_metrics(res):
@@ -152,12 +161,20 @@ class CliResourceCreate(show.ShowOne):
         return parser
 
     def _resource_from_args(self, parsed_args, update=False):
+        # Get the resource type to set the correct type
+        rt_attrs = utils.get_client(self).resource_type.get(
+            name=parsed_args.resource_type)['attributes']
         resource = {}
         if not update:
             resource['id'] = parsed_args.resource_id
         if parsed_args.attribute:
             for attr in parsed_args.attribute:
                 attr, __, value = attr.partition(":")
+                attr_type = rt_attrs.get(attr, {}).get('type')
+                if attr_type == "number":
+                    value = float(value)
+                elif attr_type == "bool":
+                    value = bool(distutils.util.strtobool(value))
                 resource[attr] = value
         if (parsed_args.add_metric
            or parsed_args.create_metric
@@ -229,3 +246,20 @@ class CliResourceDelete(command.Command):
 
     def take_action(self, parsed_args):
         utils.get_client(self).resource.delete(parsed_args.resource_id)
+
+
+class CliResourceBatchDelete(show.ShowOne):
+    """Delete a batch of resources based on attribute values"""
+
+    def get_parser(self, prog_name):
+        parser = super(CliResourceBatchDelete, self).get_parser(prog_name)
+        parser.add_argument("--type", "-t", dest="resource_type",
+                            default="generic", help="Type of resource")
+        utils.add_query_argument("query", parser)
+        return parser
+
+    def take_action(self, parsed_args):
+        res = utils.get_client(self).resource.batch_delete(
+            resource_type=parsed_args.resource_type,
+            query=parsed_args.query)
+        return self.dict2columns(res)
