@@ -13,19 +13,23 @@
 
 from cliff import show
 
+from gnocchiclient import utils
+
 
 class CliStatusShow(show.ShowOne):
     """Show the status of measurements processing"""
 
     def take_action(self, parsed_args):
-        status = self.app.client.status.get()
+        status = utils.get_client(self).status.get()
 
-        nb_metric = len(status['storage']['measures_to_process'])
-        nb_measures = (
-            sum(status['storage']['measures_to_process'].values())
-        )
+        d = {
+            "storage/total number of measures to process":
+            status['storage']['summary']['measures'],
+            "storage/number of metric having measures to process":
+            status['storage']['summary']['metrics'],
+        }
 
-        return self.dict2columns({
-            "storage/total number of measures to process": nb_measures,
-            "storage/number of metric having measures to process": nb_metric,
-        })
+        if 'metricd' in status:
+            d["metricd/processors"] = status['metricd']['processors']
+
+        return self.dict2columns(d)
